// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "DawnGame.hpp"
#include "asset/assets/TextureAsset.hpp"
#include "asset/assets/TrueTypeAsset.hpp"
#include "visualnovel/ui/VisualNovelTextbox.hpp"

using namespace Dawn;

TrueTypeAsset *assetFont;
TextureAsset *assetTexture;

DawnGame::DawnGame(DawnHost *host) :
  host(host),
  renderManager(this),
  inputManager(this)
{
}

int32_t DawnGame::init() {
  this->assetManager.init();
  this->renderManager.init();

  this->scene = new Scene(this);

  auto cameraObject = this->scene->createSceneItem();
  auto camera = cameraObject->addComponent<Camera>();
  camera->transform->lookAt(glm::vec3(50, 50, 50), glm::vec3(0, 0, 0));
  
  auto canvas = UICanvas::createCanvas(this->scene);
  
  assetFont = this->assetManager.load<TrueTypeAsset>("truetype_ark");
  assetTexture = this->assetManager.load<TextureAsset>("texture_test");
  while(!assetFont->loaded || !assetTexture->loaded) {
    this->assetManager.update();
  }

  auto textbox = canvas->addElement<VisualNovelTextbox>();
  textbox->setBorder(&assetTexture->texture, glm::vec2(16, 16));
  textbox->setFont(&assetFont->font);
  textbox->setText("Lorem ipsum dolor sit amet, consectetur adipiscing elit. Phasellus leo odio, egestas nec imperdiet ac, placerat eget quam. Nam tellus justo, aliquam sed porta quis, ullamcorper in libero. Proin auctor eget elit nec rutrum. Vestibulum tincidunt sem vel nisi sagittis, sed imperdiet eros aliquet. Fusce a ultrices augue, at auctor lacus. Sed lobortis, ante vitae vehicula egestas, lorem turpis cursus dui, sit amet egestas mauris ligula non ipsum. Pellentesque scelerisque posuere lorem sit amet tempor.  Praesent ac hendrerit mi. Nulla mollis diam vitae vestibulum aliquam. Nullam metus justo, viverra sed risus eu, tincidunt sodales lacus. Quisque efficitur accumsan posuere. Aliquam posuere volutpat diam quis lacinia. Nullam blandit nulla vestibulum mi placerat varius. Proin egestas lacus nec pellentesque iaculis. Vestibulum ex metus, congue in eleifend et, scelerisque a nulla. Pellentesque cursus lectus sed arcu efficitur tincidunt. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos. Nulla a felis non velit fermentum ullamcorper.", 40);
  textbox->setTransform(
    UI_COMPONENT_ALIGN_STRETCH, UI_COMPONENT_ALIGN_END,
    glm::vec4(0, 0, 0, 0),
    0.0f
  );
  textbox
  
  return DAWN_GAME_INIT_RESULT_SUCCESS;
}

int32_t DawnGame::update(float_t delta) {
  this->assetManager.update();
  this->inputManager.update();
  this->timeManager.update(delta);

  if(this->scene != nullptr) this->scene->update();
  
  this->renderManager.update();

  return DAWN_GAME_UPDATE_RESULT_SUCCESS;
}